//-----------------------------------------------------------------------------
// File: ddenum.pas
//
// Desc: This sample demonstrates how to enumerate all of the devices and show
//       the driver information about each.
//
//
// Copyright (c) 1998-1999 Microsoft Corporation. All rights reserved.
// Ported by Vadim Bodrov, TMT Development Corp.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
uses Windows, Messages, Strings, DXLite;
{$r ddenum.res}

//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by ddenum.rc
//
const
    IDI_MAIN_ICON                 = 101;
    IDD_DRIVERINFO                = 102;
    IDC_PREV                      = 1001;
    IDC_NEXT                      = 1002;
    IDC_DEVICEID                  = 1003;
    IDC_VERSION                   = 1004;
    IDC_DESCRIPTION               = 1005;
    IDC_FILENAME                  = 1006;
    IDC_COUNT                     = 1007;
    IDC_GUID                      = 1010;
    IDC_DWVENDORID                = 1012;
    IDC_DWDEVICEID                = 1014;
    IDC_DWSUBSYS                  = 1015;
    IDC_DWREVISION                = 1016;
    IDC_RADIO_DEVICE              = 1018;
    IDC_RADIO_HOST                = 1019;
    IDC_STATIC_WHQLLEVEL          = 1020;

// Next default values for new objects
//
    APSTUDIO_INVOKED             =  1;
    APSTUDIO_READONLY_SYMBOLS    =  1;
    _APS_NO_MFC                  =  1;
    _APS_NEXT_RESOURCE_VALUE     =  103;
    _APS_NEXT_COMMAND_VALUE      =  40001;
    _APS_NEXT_CONTROL_VALUE      =  1021;
    _APS_NEXT_SYMED_VALUE        =  103;

//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------
    NAME          =  'DDEnum';
    TITLE         =  'DirectDraw Enumeration Example';

//-----------------------------------------------------------------------------
// Default settings
//-----------------------------------------------------------------------------
    MAX_DEVICES   =  16;

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
var
    g_iMaxDevices: Longint := 0;

var
  iCurrent: Longint := 0;
  dwHost: DWORD := 0;

type _TDeviceInfo = record
    DeviceInfo: TDDDEVICEIDENTIFIER2;
    DeviceInfoHost: TDDDEVICEIDENTIFIER2;
end;

var
    g_DeviceInfo: array[0..MAX_DEVICES-1] of _TDeviceInfo;

//-----------------------------------------------------------------------------
// Name: InitFail()
// Desc: This function is called if an initialization function fails
//-----------------------------------------------------------------------------
procedure InitFail(szError: PChar);
begin
    MessageBox(0, szError, TITLE, MB_OK);
end;


//-----------------------------------------------------------------------------
// Name: SetInfoDlgText()
// Desc: Update all of the text and buttons in the dialog
//-----------------------------------------------------------------------------
procedure SetInfoDlgText(hDlg: HWnd; iCurrent: Longint; dwHost: DWORD);
var
    szBuff: array[0..127] of Char;
    chBuff: array[0..127] of Char;
    lpGUID: TGUID;
    pDI: TDDDEVICEIDENTIFIER2;
begin
    if dwHost = DDGDI_GETHOSTIDENTIFIER then
        CheckRadioButton( hDlg, IDC_RADIO_DEVICE, IDC_RADIO_HOST, IDC_RADIO_HOST )
    else
        CheckRadioButton( hDlg, IDC_RADIO_DEVICE, IDC_RADIO_DEVICE, IDC_RADIO_DEVICE );

    pDI := g_DeviceInfo[iCurrent].DeviceInfo;
    if dwHost = DDGDI_GETHOSTIDENTIFIER then
        pDI := g_DeviceInfo[iCurrent].DeviceInfoHost;

    wsprintf(szBuff, StrPCopy(chBuff, 'Device information for device ' +
        IntToStr(iCurrent + 1) + ' of ' + IntToStr(g_iMaxDevices)));
    SetDlgItemText( hDlg, IDC_RADIO_DEVICE, szBuff);


    StrPCopy(szBuff, IntToHex(pDI.dwVendorId, 8));
    SetDlgItemText( hDlg, IDC_DWVENDORID, szBuff);
    StrPCopy(szBuff, IntToHex(pDI.dwDeviceId, 8));
    SetDlgItemText( hDlg, IDC_DWDEVICEID, szBuff);
    StrPCopy(szBuff, IntToHex(pDI.dwSubSysId, 8));
    SetDlgItemText( hDlg, IDC_DWSUBSYS, szBuff);
    StrPCopy(szBuff, IntToHex(pDI.dwRevision, 8));
    SetDlgItemText( hDlg, IDC_DWREVISION, szBuff);

    //Driver version:
    StrPCopy(szBuff, IntToStr(HIWORD(pDI.liDriverVersion.HighPart)) + '.' +
        IntToStr(LOWORD(pDI.liDriverVersion.HighPart)) + '.' +
        IntToStr(HIWORD(pDI.liDriverVersion.LowPart)) + '.' +
        IntToStr(LOWORD(pDI.liDriverVersion.LowPart)));
    SetDlgItemText( hDlg, IDC_VERSION, szBuff);

    //Device description and HAL filename
    SetDlgItemText( hDlg, IDC_DESCRIPTION, pDI.szDescription);
    SetDlgItemText( hDlg, IDC_FILENAME, pDI.szDriver);

    //Unique driver/device identifier:
    lpGUID := pDI.guidDeviceIdentifier;
    StrPCopy(szBuff, IntToHex(lpGUID.D1, 8) + '-' + IntToHex(lpGUID.D2, 4) + '-' +
        IntToHex(lpGUID.D3, 4) + '-' + IntToHex(lpGUID.D4[0], 2) +
        IntToHex(lpGUID.D4[1], 2) + IntToHex(lpGUID.D4[2], 2) +
        IntToHex(lpGUID.D4[3], 2) + IntToHex(lpGUID.D4[4], 2) +
        IntToHex(lpGUID.D4[5], 2) + IntToHex(lpGUID.D4[6], 2) +
        IntToHex(lpGUID.D4[7], 2));

    SetDlgItemText( hDlg, IDC_GUID, szBuff);


    //WHQL Level
    StrPCopy(szBuff, IntToHex(pDI.dwWHQLLevel ,8));

    SetDlgItemText( hDlg, IDC_STATIC_WHQLLEVEL, szBuff);

    // Change the state and style of the Prev and Next buttons if needed
    if 0 = iCurrent then
    begin
        // The Prev button should be disabled
        SetFocus(GetDlgItem(hDlg, IDOK));
        SendDlgItemMessage( hDlg, IDC_PREV, BM_SETSTYLE, BS_PUSHBUTTON, LONGTRUE);
        SendMessage(hDlg, DM_SETDEFID, IDOK, 0);
        EnableWindow(GetDlgItem(hDlg, IDC_PREV), FALSE);
    end else
        if IsWindowEnabled(GetDlgItem(hDlg, IDC_PREV)) = FALSE then
            EnableWindow(GetDlgItem(hDlg, IDC_PREV), TRUE);

    if iCurrent >= (g_iMaxDevices - 1) then
    begin
        // The Next button should be disabled
        SetFocus(GetDlgItem(hDlg, IDOK));
        SendDlgItemMessage( hDlg, IDC_NEXT, BM_SETSTYLE, BS_PUSHBUTTON, LONGTRUE);
        SendMessage(hDlg, DM_SETDEFID, IDOK, 0);
        EnableWindow(GetDlgItem(hDlg, IDC_NEXT), FALSE);
    end else
        if IsWindowEnabled(GetDlgItem(hDlg, IDC_NEXT)) = FALSE then
            EnableWindow(GetDlgItem(hDlg, IDC_NEXT), TRUE);
end;

//-----------------------------------------------------------------------------
// Name: InfoDlgProc()
// Desc: The dialog window proc
//-----------------------------------------------------------------------------
function InfoDlgProc conv arg_stdcall (hDlg: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): HResult;
begin
    case uMsg of
        WM_INITDIALOG:
            begin
                // Setup the first devices text
                SetInfoDlgText(hDlg, iCurrent, dwHost);
                Result := LONGTRUE;
                exit;
            end;
        WM_COMMAND:
            case HIWORD(wParam) of
                BN_CLICKED:
                    case LOWORD(wParam) of
                        IDOK, IDCANCEL:
                            begin
                                EndDialog(hDlg, LONGTRUE);
                                Result := LONGTRUE;
                                exit;
                             end;
                        IDC_PREV:
                            begin
                            // Show the previous device
                                if iCurrent = 1 then
                                    iCurrent -:= 1;
                                SetInfoDlgText(hDlg, iCurrent, dwHost);
                            end;
                        IDC_NEXT:
                            begin
                            // Show the next device
                                if iCurrent < g_iMaxDevices then
                                    iCurrent +:= 1;
                                SetInfoDlgText(hDlg, iCurrent, dwHost);
                            end;
                        IDC_RADIO_HOST:
                            begin
                                dwHost := DDGDI_GETHOSTIDENTIFIER;
                                SetInfoDlgText(hDlg, iCurrent, dwHost);
                            end;
                        IDC_RADIO_DEVICE:
                            begin
                                dwHost := 0;
                                SetInfoDlgText(hDlg, iCurrent, dwHost);
                            end;
                    end;
            end;
    end;

    Result := LONGFALSE;
end;




//-----------------------------------------------------------------------------
// Name: DDEnumCallbackEx()
// Desc: This callback gets the information for each device enumerated
//-----------------------------------------------------------------------------
function DDEnumCallbackEx conv arg_stdcall(lpGUID: PGUID; pDescription, pName: LPSTR;
     pContext: Pointer; hm: THandle): LONGBOOL;
var
    pDD: PPDirectDraw;
    pDD7: PPDirectDraw7 := nil;
    hRet: HResult;
begin
    // Create the main DirectDraw object
    hRet := DirectDrawCreate(lpGUID, pDD, nil);
    if hRet <> DD_OK then
    begin
        InitFail('DirectDrawCreate FAILED');
        Result := DDENUMRET_CANCEL;
        exit;
    end;

    // Fetch DirectDraw4 interface
    hRet := pDD^^.QueryInterface^(pDD, IID_IDirectDraw7, pDD7);
    if hRet <> DD_OK then
    begin
        InitFail('QueryInterface FAILED');
        Result := DDENUMRET_CANCEL;
        exit;
    end;

    // Get the device information and save it
    hRet := pDD7^^.GetDeviceIdentifier^(pDD7, g_DeviceInfo[g_iMaxDevices].DeviceInfo,0);
    hRet := pDD7^^.GetDeviceIdentifier^(pDD7, g_DeviceInfo[g_iMaxDevices].DeviceInfoHost,DDGDI_GETHOSTIDENTIFIER);

    // Finished with the DirectDraw object, so release it
    if pDD7 <> nil then
       pDD7^^.Release^(pDD7);

    // Bump to the next open slot or finish the callbacks if full
    if g_iMaxDevices < MAX_DEVICES then
        g_iMaxDevices +:= 1
    else
    begin
        Result := DDENUMRET_CANCEL;
        exit;
    end;
    Result := DDENUMRET_OK;
end;


//-----------------------------------------------------------------------------
// Name: DDEnumCallback()
// Desc: Old style callback retained for backwards compatibility
//-----------------------------------------------------------------------------
function DDEnumCallback conv arg_stdcall (lpGUID: PGUID; pDescription, pName: LPSTR;
    context: Pointer): LONGBOOL;
begin
    Result := DDEnumCallbackEx(lpGUID, pDescription, pName, context, 0);
end;

{$w-}
type
    TDirectDrawEnumerateEx = ^function conv arg_stdcall (lpCallback: Pointer; lpContext:
        Pointer; dwFlags: DWORD): HResult;
{$w+}

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything and calls
//       DirectDrawEnumerateEx() to get all of the device info.
//-----------------------------------------------------------------------------
var
    pDirectDrawEnumerateEx: TDirectDrawEnumerateEx;
    hDDrawDLL: HModule;
begin
    // Do a GetModuleHandle and GetProcAddress in order to get the
    // DirectDrawEnumerateEx function
    hDDrawDLL := GetModuleHandle('DDRAW');
    if hDDrawDLL = 0 then
    begin
        InitFail('LoadLibrary() FAILED');
        halt(-1);
    end;
    pDirectDrawEnumerateEx := GetProcAddress(hDDrawDLL, 'DirectDrawEnumerateExA');
    if pDirectDrawEnumerateEx <> nil then
    begin
        pDirectDrawEnumerateEx^(@DDEnumCallbackEx, nil,
                                DDENUM_ATTACHEDSECONDARYDEVICES or
                                DDENUM_DETACHEDSECONDARYDEVICES or
                                DDENUM_NONDISPLAYDEVICES);
    end else
    begin
        // Old DirectDraw, so do it the old way
        DirectDrawEnumerate(@DDEnumCallback, nil);
    end;

    if 0 = g_iMaxDevices then
    begin
        InitFail('No devices to enumerate.');
        halt(-1);
    end;

    // Bring up the dialog to show all the devices
    DialogBox(hInstance, MAKEINTRESOURCE(IDD_DRIVERINFO), GetDesktopWindow, @InfoDlgProc);
end.
